package com.sharix.sportsmanfriend.repository

import android.os.StrictMode
import android.util.Log
import androidx.lifecycle.MutableLiveData
import org.jivesoftware.smack.AbstractXMPPConnection
import org.jivesoftware.smack.packet.Message
import org.jivesoftware.smack.tcp.XMPPTCPConnection
import org.jivesoftware.smack.tcp.XMPPTCPConnectionConfiguration
import org.jivesoftware.smackx.mam.MamManager
import org.jivesoftware.smackx.muc.MultiUserChat
import org.jivesoftware.smackx.muc.MultiUserChatManager
import org.jxmpp.jid.EntityBareJid
import org.jxmpp.jid.impl.JidCreate
import org.jxmpp.jid.parts.Resourcepart
import java.net.InetAddress
import java.net.URL

class XMPPRepository {
    private val TAG:String = XMPPRepository::class.java.simpleName
    private lateinit var connection: AbstractXMPPConnection
    private var manager: MultiUserChatManager? = null
    private var myJid: EntityBareJid = JidCreate.entityBareFrom("99999999999@ej.sharix-app.org")
    var muc: MultiUserChat? = null
    private var listMessage: ArrayList<Message>? = null
    var messagesLiveData = MutableLiveData<ArrayList<Message>>().apply {
        value = arrayListOf()
    }
    var messageRoom1 = MutableLiveData<ArrayList<Message>>().apply {
        value = arrayListOf()
    }
    var messageRoom2 = MutableLiveData<ArrayList<Message>>().apply {
        value = arrayListOf()
    }
    var messageRoom3 = MutableLiveData<ArrayList<Message>>().apply {
        value = arrayListOf()
    }
    init {
        try {
            val policy = StrictMode.ThreadPolicy.Builder().permitAll().build()
            StrictMode.setThreadPolicy(policy)
            val urlIP = URL("https://" + myJid.domain)
            val ip = InetAddress.getByName(urlIP.host)
            val config: XMPPTCPConnectionConfiguration = XMPPTCPConnectionConfiguration.builder()
                .setXmppDomain(myJid.domain.toString())
                .setHost(ip.hostAddress)
                .build()
            connection = XMPPTCPConnection(config)
            Log.e(TAG, "XMPP repos init")
        }catch (e: Exception){
            Log.e(TAG, e.message.toString())
        }

    }

    fun login(jid:String, password:String){
        try {
            myJid = JidCreate.entityBareFrom(jid)
            connect()
            connection.login(
                myJid.toString().split("@")[0],
                password
            )
            manager = MultiUserChatManager.getInstanceFor(connection)
            Log.i(TAG, "XMPP successfully login")
        }catch (e: Exception){
            Log.e(TAG, e.message.toString())
        }

    }
    fun messageListener(room: String, message_count: Int, idRoom: Int = 0) {
        try {
            muc = connectToRoom(room)
            listMessage = getRoomMessage(message_count, muc!!)
            messagesLiveData.value = listMessage!!
            muc!!.addMessageListener {
                if(!it.body.isNullOrEmpty()){
                    listMessage!!.add(it)
                    Log.e(TAG, idRoom.toString())
                    when (idRoom) {
                        0 -> messagesLiveData.postValue(listMessage)
                        1 -> messageRoom1.postValue(listMessage)
                        2 -> messageRoom2.postValue(listMessage)
                        3 -> messageRoom3.postValue(listMessage)
                    }
                }
            }
            Log.i(TAG, "XMPP successfully add message listener")
        }catch (e: Exception){
            Log.e(TAG, e.message.toString())
        }
    }
    fun removeAllListener(rooms: ArrayList<String>){
        try {
            for (room in rooms){
                muc = connectToRoom(room)
                listMessage = getRoomMessage(0, muc!!)
                messagesLiveData.value = listMessage!!
                muc!!.removeMessageListener {  }
                //muc!!.addMessageListener {}
            }

            Log.i(TAG, "XMPP successfully add message listener")
        }catch (e: Exception){
            Log.e(TAG, e.message.toString())
        }
    }
    fun sendMessage(msg: String){
        try {
            muc!!.sendMessage(msg)
            Log.i(TAG, "XMPP successfully send message")
        }catch (e: Exception){
            Log.e(TAG, e.message.toString())
        }
    }
    fun connect(){
        try {
            if(!connection.isConnected){
                connection.connect()
            }
            Log.i(TAG, "XMPP successfully connected")
        }catch (e: Exception){
            Log.e(TAG, e.message.toString())
        }
    }
    fun connectToRoom(room: String) : MultiUserChat?{

        return try {
            val room = JidCreate.entityBareFrom(room)
            val muc: MultiUserChat = manager!!.getMultiUserChat(room)
            val nickname = Resourcepart.from(myJid.toString())
            return if(!muc.isJoined){
                Log.i(TAG, "XMPP successfully connect to Room")
                muc.join(nickname)
                muc
            }else{
                Log.i(TAG, "XMPP error connect to Room")
                muc
            }
        }catch (e: Exception){
            Log.e(TAG, e.message.toString())
            null
        }
    }
    private fun getRoomMessage(message_count: Int, muc_conn: MultiUserChat):ArrayList<Message>? {
        return try {
            val mamManager = MamManager.getInstanceFor(muc_conn)
            val mamQueryArgs = MamManager.MamQueryArgs.builder()
                .setResultPageSizeTo(message_count)
                .queryLastPage()
                .build()
            val mamQuery = mamManager.queryArchive(mamQueryArgs)
            var arr = arrayListOf<Message>()
            arr.addAll(mamQuery.messages)
            arr
        } catch (e: Exception) {
            Log.e(TAG, e.message.toString())
            null
        }
    }

    fun logout() {
        connection.disconnect()
    }
}