package com.sharix.sportsmanfriend.view.fragments;

import android.Manifest;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageButton;

import androidx.annotation.NonNull;
import androidx.core.app.ActivityCompat;
import androidx.fragment.app.Fragment;

import com.google.android.gms.location.FusedLocationProviderClient;
import com.google.android.gms.location.LocationServices;
import com.sharix.sportsmanfriend.MapHelperClass;
import com.sharix.sportsmanfriend.OnBackPressedListener;
import com.sharix.sportsmanfriend.R;

import org.osmdroid.views.MapView;

public class JustMapFragment extends Fragment implements OnBackPressedListener {

    private View view;
    private final int REQUEST_PERMISSIONS_REQUEST_CODE = 1, PERMISSIONS_REQUEST_ACCESS_FINE_LOCATION = 2;
    private double latitude, longitude;
    private MapView map = null;
    private ImageButton btnZoomIn, btnZoomOut, btnMyLocation, btnBack;
    private FusedLocationProviderClient fusedLocationProviderClient;

    public JustMapFragment(double _latitude, double _longitude) {
        latitude = _latitude;
        longitude = _longitude;
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        view = inflater.inflate(R.layout.fragment_just_map, container, false);

        btnZoomIn = view.findViewById(R.id.btn_zoomIn);
        btnZoomOut = view.findViewById(R.id.btn_zoomOut);
        btnMyLocation = view.findViewById(R.id.btnMyLocation);
        btnBack = view.findViewById(R.id.btnBack);
        map = view.findViewById(R.id.map);

        fusedLocationProviderClient = LocationServices.getFusedLocationProviderClient(requireActivity());

        MapHelperClass.configureMap(requireActivity(), map, latitude, longitude, 15);

        addClickEventToAllViews();

        return view;
    }

    private void addClickEventToAllViews() {
        btnZoomIn.setOnClickListener(v -> MapHelperClass.zoomIn(map));

        btnZoomOut.setOnClickListener(v -> MapHelperClass.zoomOut(map));

        btnMyLocation.setOnClickListener(v -> {
            if (ActivityCompat.checkSelfPermission(requireActivity(), Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED
                    && ActivityCompat.checkSelfPermission(requireActivity(), Manifest.permission.ACCESS_COARSE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
                String[] permission = {Manifest.permission.ACCESS_FINE_LOCATION, Manifest.permission.ACCESS_COARSE_LOCATION};
                requestPermissions(permission, PERMISSIONS_REQUEST_ACCESS_FINE_LOCATION);
            }

            fusedLocationProviderClient.getLastLocation()
                    .addOnSuccessListener(requireActivity(), location -> {
                        if (location != null) {
                            MapHelperClass.setCenterOfMap(map, location.getLatitude(), location.getLongitude());
                            MapHelperClass.setZoomOfMap(map, 18);
                        }
                    });

        });

        btnBack.setOnClickListener(v -> onBackPressed() );
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        switch (requestCode) {
            case REQUEST_PERMISSIONS_REQUEST_CODE: {
                if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                    requireActivity().recreate();
                }
            }
            case PERMISSIONS_REQUEST_ACCESS_FINE_LOCATION: {
                if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                    requireActivity().recreate();
                }
            }
        }
    }

    @Override
    public void onBackPressed() {
        requireActivity().getSupportFragmentManager().popBackStack();
    }
}