package com.sharix.sportsmanfriend.view.fragments

import android.content.SharedPreferences
import android.os.Bundle
import android.util.Log
import androidx.fragment.app.Fragment
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Toast
import androidx.lifecycle.ViewModelProvider
import androidx.lifecycle.lifecycleScope
import androidx.navigation.fragment.findNavController
import com.sharix.sportsmanfriend.R
import com.sharix.sportsmanfriend.SharedPreferencesHelperClass
import com.sharix.sportsmanfriend.databinding.FragmentSupportBinding
import com.sharix.sportsmanfriend.viewmodel.XMPPViewModel
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch

class SupportFragment : Fragment() {
    private lateinit var xmppViewModel: XMPPViewModel
    private var _binding: FragmentSupportBinding? = null
    private val binding get() = _binding!!
    private var pref: SharedPreferences? = null

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        _binding = FragmentSupportBinding.inflate(inflater, container, false)
        xmppViewModel = ViewModelProvider(this@SupportFragment)[XMPPViewModel::class.java]
        val jid = SharedPreferencesHelperClass.getStringVar(activity, "userLogin") + "@ej.sharix-app.org";
        val pass = SharedPreferencesHelperClass.getStringVar(activity, "userPassword");
        Toast.makeText(activity, "$jid $pass", Toast.LENGTH_LONG).show()
        val room1 = "room1@chat.ej.sharix-app.org"
        val room2 = "room2@chat.ej.sharix-app.org"
        val room3 = "order001@chat.msg.sharix-app.org"
        if(xmppViewModel.roomMessage1.value.isNullOrEmpty()
            || xmppViewModel.roomMessage2.value.isNullOrEmpty()
            || xmppViewModel.roomMessage3.value.isNullOrEmpty()){
            val value = lifecycleScope.launch(Dispatchers.Main) {
                xmppViewModel.addRoomListener(jid!!, pass!!,
                    arrayListOf(room1!!, room2!!, room3!!))
                Log.e("MAIN", "Out from corutine")
            }
            value.invokeOnCompletion {
                Log.e("MAIN", value.toString())
                roomsSetClickListeners(room1!!, room2!!, room3!!)
                addRoomListeners()
            }
        }

        return binding.root
    }
    private fun roomsSetClickListeners(room1:String, room2:String, room3:String){
        binding.supportChat.setOnClickListener {
            activity?.intent?.putExtra("room", room1)
            activity?.intent?.putExtra("room_name", "Чат службы техподдержки")
            loadFragment(ChatFragment())
        }
        binding.serviceBot.setOnClickListener {
            activity?.intent?.putExtra("room", room2)
            activity?.intent?.putExtra("room_name", "Чат сервисного бота")
            loadFragment(ChatFragment())
        }
        binding.providerChat.setOnClickListener {
            activity?.intent?.putExtra("room", room3)
            activity?.intent?.putExtra("room_name", "Чат поставщика услуг")
            loadFragment(ChatFragment())
        }
    }
    private fun addRoomListeners(){
        xmppViewModel.roomMessage1.observe(viewLifecycleOwner){
            for(item in it){
                binding.supportLastMsg.text = item.body
            }
        }
        xmppViewModel.roomMessage2.observe(viewLifecycleOwner){
            for(item in it){
                binding.serviceBotLastMsg.text = item.body
            }
        }
        xmppViewModel.roomMessage3.observe(viewLifecycleOwner){
            for(item in it){
                binding.providerLastMsg.text = item.body
            }
        }
    }

    private fun loadFragment(fragment: Fragment){
        val transaction = activity?.supportFragmentManager?.beginTransaction()
        transaction?.replace(R.id.frameWindow, fragment)
        transaction?.addToBackStack(null)
        transaction?.commit()
    }
}